@@ -46,6 +46,7 @@
 \\unit
 \\unfoldRepeats
 \\undo
+\\undertie
 \\underline
 \\unaCorda
 \\unHideNotes
@@ -89,6 +90,7 @@
 \\tieDotted
 \\tieDashed
 \\tieDashPattern
+\\tie
 \\thumb
 \\textSpannerUp
 \\textSpannerNeutral
@@ -333,6 +335,7 @@
 \\pad-markup
 \\pad-around
 \\p
+\\overtie
 \\overrideTimeSignatureSettings
 \\overrideProperty
 \\override-lines
@@ -576,6 +579,7 @@
 \\drummode
 \\drumStyleTable
 \\drumPitchTable
+\\draw-squiggle-line
 \\draw-line
 \\draw-hline
 \\draw-dotted-line
